<?php

declare(strict_types=1);

namespace Drupal\ui_patterns_blocks\Plugin\Block;

use Drupal\Core\Block\Attribute\Block;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\ui_patterns\Form\ComponentFormBuilderTrait;
use Drupal\ui_patterns_blocks\Plugin\Derivative\ComponentBlock as DerivativeComponentBlock;

/**
 * Provides a component block.
 */
#[Block(
  id: "ui_patterns",
  admin_label: new TranslatableMarkup("Component (UI Patterns)"),
  category: new TranslatableMarkup("UI Patterns"),
  deriver: DerivativeComponentBlock::class
)]
class ComponentBlock extends BlockBase {

  use ComponentFormBuilderTrait;

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + $this->getComponentFormDefault();
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    return $this->buildComponentRenderable($this->getDerivativeId());
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    return $this->buildComponentsForm($form_state, [], $this->getDerivativeId());
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->submitComponentsForm($form_state);
  }

}
