<?php

namespace Drupal\ui_patterns_field_formatters\Plugin\Derivative;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Field\FormatterPluginManager;
use Drupal\Core\TypedData\DataDefinitionInterface;
use Drupal\Core\TypedData\TypedDataManagerInterface;
use Drupal\ui_patterns\PropTypePluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * From fields to UI patterns source plugin definitions for slots.
 *
 * @internal
 *   Plugin derivers are internal.
 */
class FieldPropertiesSourceSlotDeriver extends FieldPropertiesSourceDeriverBase {

  /**
   * Constructs new FieldBlockDeriver.
   *
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   The entity field manager.
   * @param \Drupal\Core\TypedData\TypedDataManagerInterface $typedDataManager
   *   The typed data manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\ui_patterns\PropTypePluginManager $propTypePluginManager
   *   The ui patterns prop type plugin manager.
   * @param \Drupal\Core\Field\FormatterPluginManager $formatterManager
   *   The formatter manager.
   */
  public function __construct(
    EntityFieldManagerInterface $entityFieldManager,
    TypedDataManagerInterface $typedDataManager,
    EntityTypeManagerInterface $entityTypeManager,
    PropTypePluginManager $propTypePluginManager,
    protected FormatterPluginManager $formatterManager,
  ) {
    parent::__construct($entityFieldManager, $typedDataManager, $entityTypeManager, $propTypePluginManager);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $container->get('entity_field.manager'),
      $container->get('typed_data_manager'),
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.ui_patterns_prop_type'),
      $container->get('plugin.manager.field.formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitionsForField($derivative_for_this_field, FieldStorageDefinitionInterface $field_storage_definition) {
    $options = $this->formatterManager->getOptions($field_storage_definition->getType());
    if (!empty($options)) {
      $derivative = array_merge(
        $derivative_for_this_field,
        [
          "label" => t("Field formatter"),
          "description" => t("Output of an other field formatter."),
          // For now, only for slots.
          "prop_types" => ["slot"],
        ]
      );
      $derivative['ui_patterns_field_formatter'] = array_merge($derivative['ui_patterns_field_formatter'], [
        'field_formatter' => TRUE,
      ]);
      $this->derivatives[$derivative['id']] = $derivative;
    }
    parent::getDerivativeDefinitionsForField($derivative_for_this_field, $field_storage_definition);
  }

  /**
   * {@inheritdoc}
   */
  protected function getPropTypesDeriver(DataDefinitionInterface $property_definition): array {
    return $this->propTypePluginManager->getAllPropTypeByTypedData('slot');
  }

}
