<?php

declare(strict_types=1);

namespace Drupal\ui_patterns_field_formatters\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;

/**
 * Plugin implementation of the 'component_each' formatter.
 *
 * Field types are altered in
 * ui_patterns_field_formatters_field_formatter_info_alter().
 *
 * @FieldFormatter(
 *   id = "ui_patterns_each",
 *   label = @Translation("Component for each (UI Patterns)"),
 * )
 */
class ComponentEachFormatter extends ComponentFormatterBase {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $build = [];
    for ($field_item_index = 0; $field_item_index < $items->count(); $field_item_index++) {
      // Set the field index and lang code for the component,
      // we keep the items untouched.
      $this->setComponentSettingsExtra([
        'field_index' => $field_item_index,
        'lang_code' => $langcode,
        'items' => $items,
      ]);
      $build[] = $this->buildComponentRenderable($this->getComponentConfiguration()['component_id'], $this->getComponentSourceContexts());
    }

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public static function isApplicable(FieldDefinitionInterface $field_definition) {
    $cardinality = $field_definition->getFieldStorageDefinition()->getCardinality();
    return (parent::isApplicable($field_definition) && (
        ($cardinality === FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED) ||
        ($cardinality > 1)));
  }

}
