<?php

declare(strict_types=1);

namespace Drupal\ui_patterns_field_formatters\Plugin\UiPatterns\Source;

use Drupal\Core\Plugin\Context\ContextDefinition;

/**
 * Plugin implementation of the field_label source.
 *
 * @Source(
 *   id = "field_label",
 *   label = @Translation("Field label"),
 *   description = @Translation("Field label source plugin."),
 *   prop_types = {
 *     "string"
 *   }
 * )
 */
class FieldLabelSource extends FieldSourceBase {

  /**
   * {@inheritdoc}
   */
  public function getContextDefinitions() {
    $plugin_type_context = new ContextDefinition('string');
    $plugin_type_context
      ->setRequired()
      ->addConstraint('AllowedValues', ['field_formatter']);
    return [
      'plugin_type' => $plugin_type_context,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getPropValue(): mixed {
    $field_definition = $this->getFieldDefinition();
    if (!$field_definition) {
      return NULL;
    }
    return $field_definition->getLabel();
  }

}
