<?php

declare(strict_types=1);

namespace Drupal\ui_patterns_field_formatters\Plugin\UiPatterns\Source;

/**
 * Plugin implementation of the prop source.
 *
 * @Source(
 *   id = "field_properties_source_prop",
 *   label = @Translation("Field Property (Props)"),
 *   description = @Translation("Field property source plugin for props."),
 *   deriver =
 *   "Drupal\ui_patterns_field_formatters\Plugin\Derivative\FieldPropertiesSourcePropDeriver",
 * )
 */
class FieldPropertySourceProp extends FieldPropertySourceBase {

  /**
   * {@inheritdoc}
   */
  public function getPropValue(): mixed {
    $items = $this->getEntityFieldItemList();
    // Get field index or 0 (first if formatter all).
    $delta = $this->getConfiguredSettingsExtra("field_index") ?? 0;
    $lang_code = $this->getConfiguredSettingsExtra("lang_code") ?? "und";
    $property = $this->getCustomPluginMetadata('property');
    if (empty($property)) {
      return NULL;
    }
    /** @var \Drupal\Core\Field\FieldItemInterface $field_item_at_delta */
    $field_item_at_delta = $items->get($delta);
    if (!$field_item_at_delta) {
      return NULL;
    }
    $property_value = $this->extractPropertyValue($field_item_at_delta, $property, $lang_code);
    return $this->transTypeProp($property_value, $this->propId);
  }

  /**
   * Trans-type the value to be valid inside SDC.
   *
   * @param mixed $value
   *   The value to trans-type.
   * @param string $prop_id
   *   The prop drupal id.
   *
   * @return bool|float|int
   *   The value converted.
   *
   * @todo this part will be move into UI patterns.
   */
  private function transTypeProp(mixed $value, string $prop_id): mixed {
    return match ($prop_id) {
      'decimal', 'integer', 'duration' => (int) $value,
      'float' => (float) $value,
      'boolean' => (boolean) $value,
      default => $value,
    };
  }

}
