<?php

declare(strict_types=1);

namespace Drupal\ui_patterns_field_formatters\Plugin\UiPatterns\Source;

use Drupal\Core\Field\FieldItemListInterface;

/**
 * Plugin implementation of the slot source.
 *
 * @Source(
 *   id = "field_properties_source_slot",
 *   label = @Translation("Field Property (Slots)"),
 *   description = @Translation("Field property source plugin for slot."),
 *   deriver =
 *   "Drupal\ui_patterns_field_formatters\Plugin\Derivative\FieldPropertiesSourceSlotDeriver",
 * )
 */
class FieldPropertySourceSlot extends FieldFormatterFormBase {

  /**
   * {@inheritdoc}
   */
  public function getPropValue(): mixed {
    $items = $this->getEntityFieldItemList();
    if (!$items instanceof FieldItemListInterface) {
      return [];
    }
    $field_index = $this->getConfiguredSettingsExtra('field_index') ?? NULL;
    // Render using a display formater.
    if ($this->getCustomPluginMetadata('field_formatter')) {
      return parent::getPropValue();
    }
    $property = $this->getCustomPluginMetadata('property');
    if (empty($property)) {
      return [];
    }
    $lang_code = $this->getConfiguredSettingsExtra("lang_code") ?? 'und';
    return $this->getDataItems($items, $property, $lang_code, $field_index);
  }

  /**
   * Get Data items.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface $items
   *   Items.
   * @param string $property
   *   Property.
   * @param string $lang_code
   *   Lang code.
   * @param int|null $field_index
   *   Field index.
   *
   * @return mixed
   *   Items.
   *
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  private function getDataItems(FieldItemListInterface $items, $property, $lang_code, $field_index = NULL): mixed {
    $returned = [];
    for ($delta = 0; $delta < $items->count(); $delta++) {
      /** @var \Drupal\Core\Field\FieldItemInterface $fieldValue */
      $fieldValue = $items->get($delta);
      if ((NULL !== $field_index) && $delta != $field_index) {
        continue;
      }
      $returned[] = ['#markup' => $this->extractPropertyValue($fieldValue, $property, $lang_code)];
    }
    return $returned;
  }

}
