<?php

namespace Drupal\Tests\ui_patterns_field_formatters\Kernel;

use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\KernelTests\KernelTestBase;
use Drupal\node\Entity\NodeType;

/**
 * Defines a base class for node access kernel tests.
 */
abstract class UIPatternsFieldFormattersTestBase extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'user',
    'system',
    'sdc',
    'field',
    'text',
    'filter',
    'ui_patterns',
    'ui_patterns_field_formatters',
    'node',
  ];

  /**
   * The plugin deriver mock definition.
   *
   * @var array
   */
  protected array $mockUIPatternFFExpectedDefinitions;

  /**
   * Source plugin manager.
   *
   * @var \Drupal\ui_patterns\SourcePluginManager
   */
  protected $uiPatternsSourceManager;

  /**
   * Bundle.
   *
   * @var string
   */
  protected string $bundle;

  /**
   * Property.
   *
   * @var string
   */
  protected string $property;

  /**
   * Entity Type.
   *
   * @var \Drupal\node\NodeTypeInterface
   */
  protected $entityType;

  /**
   * Field.
   *
   * @var \Drupal\field\Entity\FieldConfig
   */
  protected $field;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installSchema('user', 'users_data');
    $this->installEntitySchema('user');

    $this->setUpNode();
  }

  /**
   * Set up the node with an field body.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  private function setUpNode() {
    $this->installEntitySchema('node');
    $this->installConfig(['field', 'node']);
    $this->bundle = 'page';
    $this->property = 'value';
    $this->uiPatternsSourceManager = \Drupal::service('plugin.manager.ui_patterns_source');
    $type = NodeType::load($this->bundle);
    if (!$type) {
      $type = NodeType::create([
        'name' => 'Basic page',
        'type' => $this->bundle,
      ]);
      $type->save();
    }
    $this->entityType = $type;

    $this->field = node_add_body_field($type);

    // Set body as multiple.
    $field_storage_body = FieldStorageConfig::loadByName('node', 'body');
    $field_storage_body->setCardinality(FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
  }

}
