<?php

declare(strict_types=1);

namespace Drupal\ui_patterns_views\Plugin\UiPatterns\Source;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\Context\ContextDefinition;

/**
 * Plugin base source to handle the view group title.
 *
 * @Source(
 *   id = "view_group_title",
 *   label = @Translation("View group title"),
 *   description = @Translation("Views group title"),
 *   prop_types = {
 *     "string",
 *     "slot"
 *   },
 * )
 */
class ViewGroupTitleSource extends ViewsStyleSourceBase {

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $form = parent::settingsForm($form, $form_state);
    // Add to label grouping title information.
    $style_options = $form_state->getValue('style_options');
    if (empty($style_options['grouping']) || !is_array($style_options['grouping'])) {
      return $form;
    }
    $group_title = [];
    foreach ($style_options['grouping'] as $group) {
      if (empty($group['field'])) {
        continue;
      }
      $group_title[] = $group['field'];
    }
    if (!empty($group_title)) {
      $group_title = implode(',', $group_title);
      $form['label_map']['#title'] .= $this->t('(Grouped by: @title)', ['@title' => $group_title]);
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getPropValue(): mixed {
    $group = $this->getSettingsFromConfiguration('group');
    if (empty($group['#title'])) {
      return ['#markup' => ''];
    }
    return $this->renderOuput($group['#title']);
  }

  /**
   * {@inheritdoc}
   */
  public function getContextDefinitions() {
    $context_definition = parent::getContextDefinitions();
    // Context for views group title.
    $group_title_context = new ContextDefinition('boolean');
    $group_title_context
      ->setRequired()
      ->addConstraint('AllowedValues', [TRUE]);
    $context_definition['group_title'] = $group_title_context;
    return $context_definition;
  }

}
