<?php

declare(strict_types=1);

namespace Drupal\Tests\ui_patterns\Unit;

use Drupal\Component\Serialization\Yaml;
use Drupal\Tests\UnitTestCase;
use Drupal\ui_patterns\SchemaManager\Canonicalizer;

/**
 * @coversDefaultClass \Drupal\ui_patterns\SchemaManager\Canonicalizer
 *
 * @group ui_patterns
 */
final class CanonicalizerTest extends UnitTestCase {

  /**
   * @covers ::canonicalize
   *
   * @dataProvider provideCanonicalizerData
   */
  public function testCanonicalize(array $schema, array $expected): void {
    $canonilizer = new Canonicalizer();
    $canonilized = $canonilizer->canonicalize($schema);
    self::assertEqualsCanonicalizing($canonilized, $expected);
  }

  /**
   * Provide data for testCanonicalize.
   */
  public function provideCanonicalizerData(): \Generator {
    $sources = Yaml::decode(file_get_contents(__DIR__ . "/../../fixtures/CanonicalizerData.yml"));
    foreach ($sources as $source) {
      foreach ($source['tests'] as $test) {
        $label = $source['label'] . ':' . $test['label'];
        yield $label => [
          $test['schema'],
          $test['expected'],
        ];
      }
    };
  }

}
